@@ -18,7 +18,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 =end
 
 # Feed2Imap version
-F2I_VERSION = '0.9.3'
+F2I_VERSION = '0.9.4'
+F2I_WARNFETCHTIME = 10
 
 require 'feed2imap/config'
 require 'feed2imap/cache'
@@ -118,6 +119,7 @@ class Feed2Imap
               end
               sparefetchers -= 1
             end
+            fetch_start = Time::now
             if feed.url
               s = HTTPFetcher::fetch(feed.url, @cache.get_last_check(feed.name))
             elsif feed.execurl
@@ -133,21 +135,32 @@ class Feed2Imap
             else
               @logger.warn("No way to fetch feed #{feed.name} !")
             end
-            if feed.filter
+            if feed.filter and s != nil
               # avoid running more than one command at the same time.
               # We need it because the called command might not be
               # thread-safe, and we need to get the right exitcode.
               mutex.lock
-              Open3::popen3(feed.filter) do |stdin, stdout|
+              # hack hack hack, avoid buffering problems
+              stdin, stdout, stderr = Open3::popen3(feed.filter)
+              inth = Thread::new do
                 stdin.puts s
                 stdin.close
-                s = stdout.read
               end
+              output = nil
+              outh = Thread::new do
+                output = stdout.read
+              end
+              inth.join
+              outh.join
+              s = output
               if $?.exitstatus != 0
                 @logger.warn("Filter command for #{feed.name} exited with status #{$?.exitstatus}. Output might be corrupted !")
               end
               mutex.unlock
             end
+            if Time::now - fetch_start > F2I_WARNFETCHTIME
+              @logger.info("Fetching feed #{feed.name} took #{(Time::now - fetch_start).to_i}s")
+            end
             sparefetchers_mutex.synchronize do
               sparefetchers += 1
               sparefetchers_cond.signal
@@ -229,14 +242,17 @@ class Feed2Imap
       @logger.info("#{f.name}: #{newitems.length} new items, #{updateditems.length} updated items.") if newitems.length > 0 or updateditems.length > 0 or @logger.level == Logger::DEBUG
       begin
         if !cacherebuild
+          fn = f.name.gsub(/[^0-9A-Za-z]/,'')
           updateditems.each do |i|
-            email = item_to_mail(i, i.cacheditem.index, true, f.name, f.include_images, f.wrapto)
+            id = "<#{fn}-#{i.cacheditem.index}@acme.com>"
+            email = item_to_mail(i, id, true, f.name, f.include_images, f.wrapto)
             f.imapaccount.updatemail(f.folder, email,
-                                     i.cacheditem.index, i.date || Time::new)
+                                     id, i.date || Time::new)
           end
           # reverse is needed to upload older items first (fixes gna#8986)
           newitems.reverse.each do |i|
-            email = item_to_mail(i, i.cacheditem.index, false, f.name, f.include_images, f.wrapto)
+            id = "<#{fn}-#{i.cacheditem.index}@acme.com>"
+            email = item_to_mail(i, id, false, f.name, f.include_images, f.wrapto)
             f.imapaccount.putmail(f.folder, email, i.date || Time::new)
           end
         end
