@@ -47,25 +47,26 @@ class String
   end
 end
 
-def item_to_mail(item, index, updated, from = 'Feed2Imap', inline_images = false, wrapto = false)
+def item_to_mail(item, id, updated, from = 'Feed2Imap', inline_images = false, wrapto = false)
   message = RMail::Message::new
   if item.creator and item.creator != ''
     if item.creator.include?('@')
       message.header['From'] = item.creator.chomp
     else
-      message.header['From'] = "#{item.creator.chomp} <feed2imap@acme.com>"
+      message.header['From'] = "=?utf-8?b?#{Base64::encode64(item.creator.chomp).gsub("\n",'')}?= <feed2imap@acme.com>"
     end
   else
-    message.header['From'] = "#{from} <feed2imap@acme.com>"
+    message.header['From'] = "=?utf-8?b?#{Base64::encode64(from).gsub("\n",'')}?= <feed2imap@acme.com>"
   end
-  message.header['To'] = "#{from} <feed2imap@acme.com>"
+  message.header['To'] = "=?utf-8?b?#{Base64::encode64(from).gsub("\n",'')}?= <feed2imap@acme.com>"
+
   if item.date.nil?
     message.header['Date'] = Time::new.rfc2822
   else
     message.header['Date'] = item.date.rfc2822
   end
   message.header['X-Feed2Imap-Version'] = F2I_VERSION if defined?(F2I_VERSION)
-  message.header['X-CacheIndex'] = "-#{index}-"
+  message.header['Message-Id'] = id
   message.header['X-F2IStatus'] = "Updated" if updated
   # treat subject. Might need MIME encoding.
   subj = item.title or (item.date and item.date.to_s) or item.link
@@ -73,41 +74,39 @@ def item_to_mail(item, index, updated, from = 'Feed2Imap', inline_images = false
     if subj.needMIME
       message.header['Subject'] = "=?utf-8?b?#{Base64::encode64(subj).gsub("\n",'')}?="
     else
-      message.header['Subject'] = subj 
+      message.header['Subject'] = subj
     end
   end
   textpart = RMail::Message::new
   textpart.header['Content-Type'] = 'text/plain; charset=utf-8; format=flowed'
   textpart.header['Content-Transfer-Encoding'] = '8bit'
-  textpart.body = item.to_text(true, wrapto)
+  textpart.body = item.to_text(true, wrapto, false)
   htmlpart = RMail::Message::new
   htmlpart.header['Content-Type'] = 'text/html; charset=utf-8'
   htmlpart.header['Content-Transfer-Encoding'] = '8bit'
   htmlpart.body = item.to_html
 
   # inline images as attachments
+  imgs = []
   if inline_images
-    message.header.set('Content-Type', 'multipart/related', 'type'=> 'multipart/alternative')
-    texthtml = RMail::Message::new
-    texthtml.header.set('Content-Type', 'multipart/alternative')
-    texthtml.add_part(textpart)
-    texthtml.add_part(htmlpart)
-    message.add_part(texthtml)
-
+    cids = []
     htmlpart.body.gsub!(/(<img[^>]+)src="(\S+?\/([^\/]+?\.(png|gif|jpe?g)))"([^>]*>)/i) do |match|
       # $2 contains url, $3 the image name, $4 the image extension
       begin
         image = Base64.encode64(HTTPFetcher::fetch($2, Time.at(0)).chomp) + "\n"
         cid = "#{Digest::MD5.hexdigest($2)}@feed2imap.acme.com"
-        imgpart = RMail::Message.new
-        imgpart.header.set('Content-ID', "<#{cid}>")
-        type = $4
-        type = 'jpeg' if type.downcase == 'jpg' # hack hack hack
-        imgpart.header.set('Content-Type', "image/#{type}", 'name' => $3)
-        imgpart.header.set('Content-Disposition', 'attachment', 'filename' => $3)
-        imgpart.header.set('Content-Transfer-Encoding', 'base64')
-        imgpart.body = image
-        message.add_part(imgpart)
+        if not cids.include?(cid)
+          cids << cid
+          imgpart = RMail::Message.new
+          imgpart.header.set('Content-ID', "<#{cid}>")
+          type = $4
+          type = 'jpeg' if type.downcase == 'jpg' # hack hack hack
+          imgpart.header.set('Content-Type', "image/#{type}", 'name' => $3)
+          imgpart.header.set('Content-Disposition', 'attachment', 'filename' => $3)
+          imgpart.header.set('Content-Transfer-Encoding', 'base64')
+          imgpart.body = image
+          imgs << imgpart
+        end
         # now to specify what to replace with
         newtag = "#{$1}src=\"cid:#{cid}\"#{$5}"
         #print "#{cid}: Replacing '#{$&}' with '#{newtag}'...\n"
@@ -117,6 +116,17 @@ def item_to_mail(item, index, updated, from = 'Feed2Imap', inline_images = false
         $& # don't modify on exception
       end
     end
+  end
+  if imgs.length > 0
+    message.header.set('Content-Type', 'multipart/related', 'type'=> 'multipart/alternative')
+    texthtml = RMail::Message::new
+    texthtml.header.set('Content-Type', 'multipart/alternative')
+    texthtml.add_part(textpart)
+    texthtml.add_part(htmlpart)
+    message.add_part(texthtml)
+    imgs.each do |i|
+      message.add_part(i)
+    end
   else
     message.header['Content-Type'] = 'multipart/alternative'
     message.add_part(textpart)
